<?php
function generate_revenue_report($start_date, $end_date, $category_id = null) {
    if (!class_exists('WooCommerce')) {
        return 'WooCommerce not found';
    }

    $currency_symbol = get_woocommerce_currency_symbol();

    $args = array(
        'type'      => 'shop_order',
        'status'    => array('wc-completed', 'wc-refunded'),
        'date_query' => array(
            'after'     => $start_date,
            'before'    => $end_date,
            'inclusive' => true,
        ),
        'limit' => -1,
    );

    $orders = wc_get_orders($args);

    $report = array();
    foreach ($orders as $order) {
        $include_order = false;

        if ($category_id) {
            $items = $order->get_items();
            foreach ($items as $item) {
                $product_id = $item->get_product_id();
                $product_cats = wc_get_product_term_ids($product_id, 'product_cat');

                if (in_array($category_id, $product_cats)) {
                    $include_order = true;
                    break;
                }
            }
        } else {
            $include_order = true;
        }

        if (!$include_order) {
            continue;
        }
        
        $date = $order->get_date_created()->date('Y-m-d');
        if (!isset($report[$date])) {
            $report[$date] = array('orders' => 0, 'gross_sales' => 0, 'returns' => 0, 'coupons' => 0, 'net_sales' => 0, 'taxes' => 0, 'total_sales' => 0);
        }

        if ($order->get_status() == 'completed' or $order->get_status() == 'processing') {
            $report[$date]['orders']++;
            $report[$date]['gross_sales'] += floatval($order->get_subtotal());
            $report[$date]['coupons'] += floatval($order->get_total_discount());
            $report[$date]['taxes'] += floatval($order->get_total_tax());
            $report[$date]['net_sales'] += floatval($order->get_total()) - floatval($order->get_total_tax()); // Monto neto sin incluir impuestos
            $report[$date]['total_sales'] += floatval($order->get_total());
        } elseif ($order->get_status() == 'refunded') {
            $report[$date]['returns'] += floatval($order->get_total());
        }
    }

    $output = "Date,Orders,Gross sales,Returns,Coupons,Net sales,Taxes,Total sales\n";
    foreach ($report as $date => $data) {
        $net_sales = $data['net_sales'] - $data['returns'];
        $total_sales = $data['total_sales'] - $data['returns'];
        $output .= "$date,{$data['orders']},{$currency_symbol}" . number_format($data['gross_sales'], 2) . ",{$currency_symbol}" . number_format($data['returns'], 2) . ",{$currency_symbol}" . number_format($data['coupons'], 2) . ",{$currency_symbol}" . number_format($net_sales, 2) . ",{$currency_symbol}" . number_format($data['taxes'], 2) . ",{$currency_symbol}" . number_format($total_sales, 2) . "\n";
    }

    return array('output' => $output, 'report' => $report, 'currency_symbol' => $currency_symbol);
}

function revenue_report_menu() {
    add_menu_page(
        'Revenue Report',
        'Revenue Report',
        'manage_options',
        'revenue-report',
        'revenue_report_page',
        'dashicons-chart-line',
        6
    );
}

add_action('admin_menu', 'revenue_report_menu');

function revenue_report_page() {
    if (!current_user_can('manage_options')) {
        return;
    }

    $categories = get_terms(array('taxonomy' => 'product_cat', 'hide_empty' => false));

    if (isset($_POST['generate_report'])) {
        $start_date = sanitize_text_field($_POST['start_date']);
        $end_date = sanitize_text_field($_POST['end_date']);
        $category_id = isset($_POST['category_id']) ? intval($_POST['category_id']) : null;
        $report_data = generate_revenue_report($start_date, $end_date, $category_id);
        $report = $report_data['report'];
        $csv_output = $report_data['output'];
        $currency_symbol = $report_data['currency_symbol'];
        ?>

        <div class="wrap">
            <h1>Generate Revenue Report</h1>
            <form method="post">
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row">Start Date</th>
                        <td><input type="date" name="start_date" required value="<?php echo esc_attr($start_date); ?>" /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">End Date</th>
                        <td><input type="date" name="end_date" required value="<?php echo esc_attr($end_date); ?>" /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Product Category</th>
                        <td>
                            <select name="category_id">
                                <option value="">All Categories</option>
                                <?php foreach ($categories as $category): ?>
                                    <option value="<?php echo esc_attr($category->term_id); ?>" <?php selected($category_id, $category->term_id); ?>><?php echo esc_html($category->name); ?></option>
                                <?php endforeach; ?>
                            </select>
                        </td>
                    </tr>
                </table>
                <?php submit_button('Generate Report', 'primary', 'generate_report'); ?>
            </form>

            <h2>Revenue Report from <?php echo esc_html($start_date); ?> to <?php echo esc_html($end_date); ?> for category <?php echo $category_id ? esc_html(get_term($category_id)->name) : 'All Categories'; ?></h2>
            <table class="wp-list-table widefat fixed striped">
                <thead>
                    <tr>
                        <th>Date</th>
                        <th>Orders</th>
                        <th>Gross sales (<?php echo esc_html($currency_symbol); ?>)</th>
                        <th>Returns (<?php echo esc_html($currency_symbol); ?>)</th>
                        <th>Coupons (<?php echo esc_html($currency_symbol); ?>)</th>
                        <th>Net sales (<?php echo esc_html($currency_symbol); ?>)</th>
                        <th>Taxes (<?php echo esc_html($currency_symbol); ?>)</th>
                        <th>Total sales (<?php echo esc_html($currency_symbol); ?>)</th>
                    </tr>
                </thead>
                <tbody>
                    <?php foreach ($report as $date => $data): ?>
                        <tr>
                            <td><?php echo esc_html($date); ?></td>
                            <td><?php echo esc_html($data['orders']); ?></td>
                            <td><?php echo esc_html(number_format($data['gross_sales'], 2)) . ' ' . esc_html($currency_symbol); ?></td>
                            <td><?php echo esc_html(number_format($data['returns'], 2)) . ' ' . esc_html($currency_symbol); ?></td>
                            <td><?php echo esc_html(number_format($data['coupons'], 2)) . ' ' . esc_html($currency_symbol); ?></td>
                            <td><?php echo esc_html(number_format($data['net_sales'] - $data['returns'], 2)) . ' ' . esc_html($currency_symbol); ?></td>
                            <td><?php echo esc_html(number_format($data['taxes'], 2)) . ' ' . esc_html($currency_symbol); ?></td>
                            <td><?php echo esc_html(number_format($data['total_sales'] - $data['returns'], 2)) . ' ' . esc_html($currency_symbol); ?></td>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>

            <form method="post" action="<?php echo esc_url(admin_url('admin-post.php')); ?>">
                <input type="hidden" name="action" value="download_csv_report">
                <input type="hidden" name="csv_output" value="<?php echo esc_attr($csv_output); ?>">
                <?php submit_button('Download CSV', 'secondary'); ?>
            </form>
        </div>

        <?php
    } else {
        ?>
        <div class="wrap">
            <h1>Generate Revenue Report</h1>
            <form method="post">
                <table class="form-table">
                    <tr valign="top">
                        <th scope="row">Start Date</th>
                        <td><input type="date" name="start_date" required /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">End Date</th>
                        <td><input type="date" name="end_date" required /></td>
                    </tr>
                    <tr valign="top">
                        <th scope="row">Product Category</th>
                        <td>
                            <select name="category_id">
                                <option value="">All Categories</option>
                                <?php foreach ($categories as $category): ?>
                                    <option value="<?php echo esc_attr($category->term_id); ?>"><?php echo esc_html($category->name); ?></option>
                                <?php endforeach; ?>
                            </select>
                        </td>
                    </tr>
                </table>
                <?php submit_button('Generate Report', 'primary', 'generate_report'); ?>
            </form>
        </div>
        <?php
    }
}

function download_csv_report() {
    if (isset($_POST['csv_output'])) {
        $csv_output = sanitize_textarea_field($_POST['csv_output']);
        
        header('Content-Type: text/csv; charset=UTF-8');
        header('Content-Disposition: attachment; filename="revenue_report.csv"');
        header('Pragma: no-cache');
        header('Expires: 0');

        // Añadir BOM para Excel
        echo "\xEF\xBB\xBF";
        
        echo $csv_output;
        exit;
    }
}
add_action('admin_post_download_csv_report', 'download_csv_report');


//CAMBIA LA FECHA DE CREACION DE LA ORDEN PARA QUE EL REPORTE SEA CORRECTO,
// ESTO SOLO SUCEDE EN LA VENTANA DE BACKEND CUANDO SE HACE CLICK EN EL BOTON UPDATE
add_action( 'save_post', 'my_save_post_function', 10, 3 );
function my_save_post_function( $post_ID) {
    
    if ( get_post_type($post_ID) == 'shop_order' )  { 
        $woo_order = wc_get_order( $post_ID );
        $woo_order_status = $woo_order->get_status();
        if ( $woo_order_status == 'processing' || $woo_order_status == 'completed' ){
            $now = new DateTime();
            $woo_order->set_date_created($now->getTimestamp());
            $woo_order->save();
        } 
    }    
}